#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Author   : pu mingzheng <pumz_1991@126.com>
# @Time     : 2023/03/14 10:48:38
# @File     : easy_tools/base/design_pattern - singleton.py
# @Software : python3.11 PyCharm
# @Desc     : TODO

from functools import wraps


def singleton(cls):
    instances = {}

    @wraps(cls)
    def get_instance(*args, **kw):
        if cls not in instances:
            instances[cls] = cls(*args, **kw)
        return instances[cls]

    return get_instance


def singleton_unique(cls):
    instances = {}

    @wraps(cls)
    def get_instance(*args, **kw):
        unique_key = "{}_{}_{}".format(cls, args, kw)
        if unique_key not in instances:
            instances[unique_key] = cls(*args, **kw)
        return instances[unique_key]

    return get_instance



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Author   : pu mingzheng <pumz_1991@126.com>
# @Time     : 2023/03/14 11:22:30
# @File     : easy_tools/base/encryption - rsa.py
# @Software : python3.11 PyCharm
# @Desc     : TODO
import base64
import hashlib
import uuid
from typing import Tuple

from Crypto import Random
from Crypto.Cipher import PKCS1_v1_5 as PKCS1_cipher
from Crypto.Hash import SHA256
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5 as PKCS1_signature


def generator_rsa(key_length_in_bits: int = 1024, filename: str = "") -> Tuple[bytes, bytes]:
    """
    生成 rsa 秘钥
    :param key_length_in_bits: Select the length of your key pair between 512 and 16384 bits
    :param filename:
    :param save:
    :return:
    """
    random_generator = Random.new().read
    rsa = RSA.generate(key_length_in_bits, random_generator)
    rsa_private_key = rsa.exportKey()
    rsa_public_key = rsa.publickey().exportKey()

    if filename:
        with open(f"{filename}_private.pem", "w") as f:
            f.write(rsa_private_key.decode())
        with open(f"{filename}_public.pem", "w") as f:
            f.write(rsa_public_key.decode())
    print(len(rsa_private_key))  # 866
    print(len(rsa_public_key))  # 271
    return rsa_private_key, rsa_public_key


def get_key(str_key):
    key = RSA.importKey(str_key)
    return key


# 公钥加密
def rsa_public_encode(password, public_key):
    cipher = PKCS1_cipher.new(RSA.importKey(public_key))
    encrypt_text = base64.b64encode(cipher.encrypt(password.encode("utf-8")))
    return encrypt_text.decode("utf-8")


# 私钥解密
def rsa_private_decode(password, private_key):
    cipher = PKCS1_cipher.new(RSA.importKey(private_key))
    decrypt_text = cipher.decrypt(base64.b64decode(password), Random.new().read)
    return decrypt_text.decode("utf-8")


# 私钥签名
def rsa_private_sign(data, private_key):
    private_key = RSA.importKey(private_key)
    signer = PKCS1_signature.new(private_key)
    digest = SHA256.new()
    digest.update(data.encode("utf8"))
    sign = signer.sign(digest)
    signature = base64.b64encode(sign)
    signature = signature.decode("utf-8")
    return signature


# 公钥验证
def rsa_public_sign(text, sign, public_key):
    public_key = RSA.importKey(public_key)
    verifier = PKCS1_signature.new(public_key)
    digest = SHA256.new()
    digest.update(text.encode("utf-8"))
    return verifier.verify(digest, base64.b64decode(sign))


# 设置 MD5 为文件名
def filename2md5(filename):
    md5 = hashlib.md5()
    md5.update(filename.encode())
    return md5.hexdigest().upper()


# 设置 uuid 为文件名
def filename2uuid(filename):
    return uuid.uuid4().hex.upper()


if __name__ == "__main__":
    generator_rsa(filename="soul")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Author   : pu mingzheng <pumz_1991@126.com>
# @Time     : 2023/03/20 16:01:13
# @File     : easy_tools - test.py
# @Software : python3.11 PyCharm
# @Desc     : TODO

import hmac
import math
import struct
import time
from abc import ABC, abstractmethod


class OTP:
    codeLength = 8
    timeSec = 30
    digest = "SHA1"

    @classmethod
    def get_code(cls, secret, time_slice=None):
        if time_slice is None:
            time_slice = math.floor(time.time() - OTP.timeSec)
            # 进行加密
        hm = hmac.new(key=secret.encode("utf-8"), msg=str(time_slice).encode("utf-8"), digestmod=OTP.digest).digest()
        # 进行二进制解包
        pack_array = struct.unpack(">L", hm[0:4])
        num = pack_array[0]
        # 补位
        num = num & 0x7FFFFFFF
        modulo = pow(10, OTP.codeLength)
        s_sum = str(num % modulo)
        s_sum = s_sum.zfill(OTP.codeLength)
        print(s_sum)
        return s_sum


class Main:
    def main(self):
        # secret由Eskyfun提供
        secret = "745a950f994b7da417cd438e5619ac7329400460"

        # 平台SDK提供token、userId参数
        token = "03c7c0ace395d80182db07ae2c30f034.1499671547.55935154"
        userId = "1000001"

        # 分割token
        tokenArray = token.split(".")
        if len(tokenArray) == 3:
            # 计数器
            times = tokenArray[1]
            # 验证码
            code = tokenArray[2]
            # 使用密钥和用户ID，拼接用户唯一密钥值
            userSecret = tokenArray[0] + userId + secret
            # 获取code
            oneCode = OTP.get_code(userSecret, times)
            # 对比验证
            if oneCode == code:
                # 成功
                print(1)
            else:
                # 失败
                print(0)
        else:
            # 失败
            print(0)


class A(ABC):
    def func(self):
        print(1)

    @abstractmethod
    def _func(self):
        print(1111111)


class B(A):
    def _func(self):
        print("B")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Author   : pu mingzheng <pumz_1991@126.com>
# @Time     : 2023/03/14 10:48:38
# @File     : easy_tools/base/encryption - translate.py
# @Software : python3.11 PyCharm
# @Desc     : TODO

from aes import AESHandler


# str translate demo
def str_translate(
    string: str = "",
    in_table: str = "123qwertyuiop456asdfghjkl789zxcvbnm ",
    out_table: str = "~!@#$%789^&*()_+-=654,.<>/;312:[]{}'",
    back: bool = False,
) -> str:
    if back:
        in_table, out_table = out_table, in_table

    translate_table = str.maketrans(in_table, out_table)

    return string.translate(translate_table)


def aes_str_translate(string: str, in_table: str, out_table: str, aes_key: str = "", back: bool = False) -> str:
    aes = AESHandler(aes_key)
    in_table = aes.decrypt_AES(in_table)
    out_table = aes.decrypt_AES(out_table)

    return str_translate(string, in_table, out_table, back)


def test_aes_str_translate(string: str, back: bool = False) -> str:
    default_in_table = "123qwertyuiop456asdfghjkl789zxcvbnm "
    default_out_table = "~!@#$%789^&*()_+-=654,.<>/;312:[]{}'"
    aes_key = "cfd71c5af6a9168d3c2d9aa5c21649fa"

    aes = AESHandler(aes_key=aes_key)
    in_table = aes.encrypt_AES(default_in_table)
    out_table = aes.encrypt_AES(default_out_table)

    return aes_str_translate(string, in_table, out_table, aes_key, back)


if __name__ == "__main__":
    pass
    # a = str_translate("show me the money")
    # b = str_translate(a, True)
    # print(a)
    # print(b)

    string = "show me the money"
    a = test_aes_str_translate(string)
    b = test_aes_str_translate(a, True)
    print(a)
    print(b)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Author   : pu mingzheng <pumz_1991@126.com>
# @Time     : 2023/03/14 11:43:46
# @File     : easy_tools/base/demo - pyhumps_demo.py
# @Software : python3.11 PyCharm
# @Desc     : TODO

import humps

humps.camelize("jack_in_the_box")  # jackInTheBox
humps.decamelize("rubyTuesdays")  # ruby_tuesdays
humps.pascalize("red_robin")  # RedRobin
humps.kebabize("white_castle")  # white-castle

array = [{"attrOne": "foo"}, {"attrOne": "bar"}]
humps.decamelize(array)  # [{"attr_one": "foo"}, {"attr_one": "bar"}]

array = [{"attr_one": "foo"}, {"attr_one": "bar"}]
humps.camelize(array)  # [{"attrOne": "foo"}, {"attrOne": "bar"}]

array = [{"attr_one": "foo"}, {"attr_one": "bar"}]
humps.kebabize(array)  # [{'attr-one': 'foo'}, {'attr-one': 'bar'}]

array = [{"attr_one": "foo"}, {"attr_one": "bar"}]
humps.pascalize(array)  # [{"AttrOne": "foo"}, {"AttrOne": "bar"}]

humps.is_camelcase("illWearYourGranddadsClothes")  # True
humps.is_pascalcase("ILookIncredible")  # True
humps.is_snakecase("im_in_this_big_ass_coat")  # True
humps.is_kebabcase("from-that-thrift-shop")  # True
humps.is_camelcase("down_the_road")  # False

humps.is_snakecase("imGonnaPopSomeTags")  # False
humps.is_kebabcase("only_got_twenty_dollars_in_my_pocket")  # False


# what about abbrevations, acronyms, and initialisms? No problem!
humps.decamelize("APIResponse")  # api_response


# # aws.py
# import humps
# import boto3
#
# def api(service, decamelize=True, *args, **kwargs):
#     service, func = service.split(":")
#     client = boto3.client(service)
#     kwargs = humps.pascalize(kwargs)
#     response = getattr(client, func)(*args, **kwargs)
#     return (depascalize(response) if decamelize else response)
#
# # usage
# api("s3:download_file", bucket="bucket", key="hello.png", filename="hello.png")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Author   : pu mingzheng <pumz_1991@126.com>
# @Time     : 2023/03/20 09:45:31
# @File     : easy_tools/base/time_translate - base.py
# @Software : python3.11 PyCharm
# @Desc     : TODO

import datetime

import pytz

default_tz = pytz.timezone("Asia/Shanghai")


def timestamp_to_datetime(timestamp: int, timezone=default_tz) -> str:
    """

    :param timestamp: 10 位时间戳
    :param timezone: 时区
    :return:
    """
    res = datetime.datetime.fromtimestamp(timestamp, timezone).strftime("%Y-%m-%d %H:%M:%S")
    return res



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Author   : pu mingzheng <pumz_1991@126.com>
# @Time     : 2023/03/14 10:48:38
# @File     : easy_tools/base/encryption - aes.py
# @Software : python3.11 PyCharm
# @Desc     : TODO

import base64

from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad


class AESHandler:
    def __init__(self, aes_key: str):
        # self.aes_key = hashlib.md5(aes_key.encode("utf-8")).hexdigest().encode()
        self.aes_key = aes_key.encode()

    def signature(self):
        ...

    def encrypt_AES(self, data: str, block_size=32):
        raw = pad(data.encode(), block_size)
        cipher = AES.new(self.aes_key, AES.MODE_ECB)
        return base64.b64encode(cipher.encrypt(raw)).decode()

    def decrypt_AES(self, data: str, block_size=32):
        enc = base64.b64decode(data)
        cipher = AES.new(self.aes_key, AES.MODE_ECB)
        return unpad(cipher.decrypt(enc), block_size).decode()


if __name__ == "__main__":
    aes_key = "cfd71c5af6a9168d3c2d9aa5c21649fa"
    aes = AESHandler(aes_key)
    s = "123456"
    en_s = aes.encrypt_AES(s)
    de_s = aes.decrypt_AES(en_s)
    print(en_s, de_s)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Author   : pu mingzheng <pumz_1991@126.com>
# @Time     : 2023/03/14 11:39:38
# @File     : easy_tools/base/demo - case_convert_demo.py
# @Software : python3.11 PyCharm
# @Desc     : TODO

import case_convert



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Author   : pu mingzheng <pumz_1991@126.com>
# @Time     : 2023/03/14 10:48:38
# @File     : easy_tools/base - common.py
# @Software : python3.11 PyCharm
# @Desc     : TODO

import random
import re
import secrets

RANDOM_STRING_CHARS = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"


def get_random_string(length: int, allowed_chars: str = RANDOM_STRING_CHARS) -> str:
    """
    Return a securely generated random string.
    The bit length of the returned value can be calculated with the formula:
        log_2(len(allowed_chars)^length)
    For example, with default `allowed_chars` (26+26+10), this gives:
      * length: 12, bit length =~ 71 bits
      * length: 22, bit length =~ 131 bits
    """
    return "".join(secrets.choice(allowed_chars) for i in range(length))


def filter_special_character(string: str) -> str:
    """
    过滤参数
    字符 Unicode 编码范围
    数字: \u0030-\u0039
    汉字: \u4e00-\u9fa5
    大写字母: \u0041-\u005a
    小写字母: \u0061-\u007a
    :param string:
    :return:
    """
    return re.sub("([^\u0030-\u0039\u0041-\u005a\u0061-\u007a])", "", string)


def random_str(string: str) -> str:
    """
    打乱字符串
    :param string:
    :return:
    """
    string_list = list(string)
    random.shuffle(string_list)
    return "".join(string_list)


if __name__ == "__main__":
    a = "@#$#@%$%(*)*_(&*(^KGJHasd"
    print(filter_special_character(a))
    print(random_str(a))
